# Copyright (C) 2006-2008 by
# Eduardo Robles Elvira <edulix@gmail.com>
# Sergio Pistone <sergio_pistone@yahoo.com.ar>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/http"
require "lyrics"

class Jamendo < Lyrics

	def Jamendo.site_host()
		return "www.jamendo.com"
	end

	def Jamendo.site_name()
		return "Jamendo"
	end

	def Jamendo.lyrics_test_data()
		return [
			Request.new( "Misantropía", "Hipócrita (Hypocrite)", "Misantropía" ),
			Request.new( "Inesperado", "Pez Gordo", "Maqueta Inesperada" ),
			Request.new( "Tirpunk", "Suicide", "Tirpunk-Újra" ),
			Request.new( "Punkamine", "Gora gerra", "Punkamine" ),
		]
	end

	def Jamendo.cleanup_artist_name( artist )
		artist = Strings.downcase( artist )
		Strings.remove_vocal_accents!( artist )
		artist.gsub!( /\[|\]|:/, "" )
		artist.gsub!( /_| |'|"|ø|&|@|\/|\*|\.|®|%|#/, " " ) # \' can also be ''
		artist.squeeze!( " " )
		artist.strip!()
		artist.gsub!( " ", "." )
		return artist
	end

	def cleanup_artist_name( artist )
		self.class.cleanup_artist_name( artist )
	end

	def build_lyrics_fetch_data( request )
		url =  "http://#{site_host()}/en/get/track/list/track-artist-album/lyricstext/plain/?"
		url << "searchterm=#{CGI.escape( request.title )}"
		url << "&artist_searchterm=#{CGI.escape( request.artist )}"
		url << "&album_searchterm=#{CGI.escape( request.album )}" if ( request.album )
		return FetchPageData.new( url )
	end

	def parse_lyrics( response, page_body )

		page_body.tr_s!( " \n\r\t", " " )

		if ( /track-artist-album/.match( response.url ) )

			md = /[?&]searchterm=([^&]+)(&?|$)/.match( response.url )
			response.title = CGI.unescape( md[1] ) if ( md )

			md = /[?&]artist_searchterm=([^&]+)(&?|$)/.match( response.url )
			response.artist = CGI.unescape( md[1] ) if ( md )

			md = /[?&]album_searchterm=([^&]+)(&?|$)/.match( response.url )
			response.album = CGI.unescape( md[1] ) if ( md )

			response.lyrics = page_body if ( ! page_body.empty?() )

		else

			return if ( ! page_body.sub!( /^.*<div id='page_content_inner'>/, "" ) )

			metadata = {}
			if ( md = /<div><div><b>([^-]+) - ([^(]+) \(([0-9]+)\) ([^<]+)<\/b><\/div><div>/.match( page_body ) )
				tags = ["artist", "album", "track", "title"]
				tags.each() { |tag| metadata[tag] = md[tags.index( tag ) + 1] }
			end

			response.artist = metadata["artist"]
			response.title = metadata["title"]
			response.album = metadata["album"]
			response.custom_data = { "track" => metadata["track"] }

			return if ( ! page_body.sub!( /^.*<\/div><div>/, "" ) )
			return if ( ! page_body.sub!( /<\/div>.*$/, "" ) )

			page_body.gsub!( /\ ?<br ?\/?> ?/i, "\n" )

			response.lyrics = page_body

		end

	end

	def suggestions( request )

		suggestions = []

		artist_page_url = "http://#{site_host()}/en/artist/#{cleanup_artist_name( request.artist )}/"

		if ( log?() )
			log( site_name().upcase() )
			log( "Retrieving SUGGESTIONS..." )
			log( request.to_s() )
			log( " - artist url: #{artist_page_url}" )
			log( "Fetching artist page... ", 0 )
		end

		response, artist_page_url = HTTP.fetch_page_get( artist_page_url, nil, 0 )

		if ( response && response.body() )
			error = response.body().include?( "has no album on <b>jamendo</b> yet" )
			if ( error && log?() )
				log( "INVALID PAGE" )
			elsif ( log?() )
				log( "OK" )
			end
			log( response.body(), 2 ) if ( verbose_log?() )
		else
			error = true
			log( "ERROR" ) if ( log?() )
		end

		if ( error )
			gs_artist = Strings.google_search_quote( request.artist )
			fl_artist_page_url = Strings.build_google_feeling_lucky_url(
				"artist \"albums on jamendo\" #{gs_artist}",
				"#{site_host()}/en"
			)
			if ( log?() )
				log( " - built feeling lucky artist url: #{fl_artist_page_url}" )
				log( "Fetching artist page... ", 0 )
			end

			response, fl_artist_page_url = HTTP.fetch_page_get( fl_artist_page_url )
			error = ! response || ! response.body()

			if ( log?() )
				if ( error )
					log( "ERROR" )
				else
					md = /<title>Jamendo : (.*)<\/title>/.match( response.body() )
					error = md == nil || (Strings.normalize( md[1] ) != Strings.normalize( request.artist ))
					if ( error )
						log( "INVALID PAGE" )
					else
						log( "OK" )
					end
					log( response.body(), 2 ) if ( verbose_log?() )
				end
			end

		end

		return suggestions if ( error )

		artist_page_body = response.body()

		log( "Parsing artist album pages...", 0 ) if ( log?() )

		# artist_body should contain the artist page, we have to iterate through the album pages excerpting the lyrics links
		artist_page_body.split( /<div class='g_album_name'>/ ).each() do |album_entry|
			if ( md = /<a href="(\/en\/album\/[0-9]+)" >([^<]+)<\/a>/.match( album_entry ) )

				album_page_url = "http://#{site_host()}#{md[1]}"
				album_name = md[2]

				log( "\n - #{album_page_url}", 0 ) if ( log?() )

				response, album_page_url = HTTP.fetch_page_get( album_page_url )
				next if ( ! response || ! response.body() )

				album_page_body = response.body()
				album_page_body.tr_s!( " \n\r\t", " " )

				album_page_body.sub!( /^.*<tbody>/, "" )
				album_page_body.sub!( /<\/tbody>.*$/, "" )

				# album_body contains the album page, we have to extract the lyrics links
				album_page_body.split( "<td class=\"title_tracks\">" ).each() do |song_entry|
					if ( md = /<a href='(\/en\/track\/[0-9]+)'> ([^<]+) <\/a>/.match( song_entry ) )
						suggestions << Suggestion.new( request.artist, md[2].strip(), "http://#{site_host()}#{md[1]}" )
					end
				end

			end
		end

		if ( log?() )
			log( suggestions.empty?() ? " NO SUGGESTIONS FOUND" : "" )
			suggestions.each() { |suggestion| log( " - #{suggestion}" ) }
		end

		return suggestions

	end

end
